import pytest
import json
from cve_checker.report_generator import ReportGenerator

@pytest.fixture
def report_generator():
    return ReportGenerator()

@pytest.fixture
def sample_vulnerabilities():
    return [
        {
            'cve': {
                'CVE_data_meta': {'ID': 'CVE-2021-1234'},
                'description': {'description_data': [{'value': 'Test vulnerability'}]}
            },
            'impact': {'baseMetricV3': {'cvssV3': {'baseScore': 7.5}}},
            'configurations': {
                'nodes': [
                    {'cpe_match': [{'vulnerable': True, 'cpe23Uri': 'cpe:2.3:a:vendor:product:version:*:*:*:*:*:*:*'}]}
                ]
            },
            'risk_score': 8.2
        }
    ]

def test_generate_json_report(report_generator, sample_vulnerabilities):
    report = report_generator.generate_report(sample_vulnerabilities, 'json')
    report_data = json.loads(report)

    assert 'generated_at' in report_data
    assert report_data['total_vulnerabilities'] == 1
    assert len(report_data['vulnerabilities']) == 1
    
    vuln = report_data['vulnerabilities'][0]
    assert vuln['cve_id'] == 'CVE-2021-1234'
    assert vuln['description'] == 'Test vulnerability'
    assert vuln['cvss_score'] == 7.5
    assert vuln['risk_score'] == 8.2
    assert vuln['affected_packages'] == ['product']

def test_generate_html_report(report_generator, sample_vulnerabilities):
    report = report_generator.generate_report(sample_vulnerabilities, 'html')

    assert '<html>' in report
    assert '<table>' in report
    assert 'CVE-2021-1234' in report
    assert 'Test vulnerability' in report
    assert '7.5' in report
    assert '8.2' in report
    assert 'product' in report

def test_unsupported_format(report_generator, sample_vulnerabilities):
    with pytest.raises(ValueError):
        report_generator.generate_report(sample_vulnerabilities, 'xml')



from typing import List, Dict
from .cve_data_fetcher import CVEDataFetcher

class VulnerabilityChecker:
    def __init__(self, cve_data_fetcher: CVEDataFetcher):
        self.cve_data_fetcher = cve_data_fetcher

    async def check_vulnerabilities(self, software_name: str, version: str) -> List[Dict]:
        cve_data = await self.cve_data_fetcher.fetch_cve_data()
        vulnerabilities = []

        for cve_item in cve_data:
            cve_info = cve_item['cve']
            affected_products = cve_info.get('affects', {}).get('vendor', {}).get('vendor_data', [])

            for product in affected_products:
                if product['product']['product_data'][0]['product_name'].lower() == software_name.lower():
                    for version_data in product['product']['product_data'][0]['version']['version_data']:
                        if self._version_is_vulnerable(version, version_data):
                            vulnerabilities.append({
                                'cve_id': cve_info['CVE_data_meta']['ID'],
                                'description': cve_info['description']['description_data'][0]['value'],
                                'affected_version': version_data
                            })

        return vulnerabilities

    def _version_is_vulnerable(self, current_version: str, version_data: Dict) -> bool:
        # This is a simplified version check. In a real-world scenario,
        # you'd want to implement a more robust version comparison logic.
        return current_version in version_data['version_value']



import aiohttp
import asyncio
import json
import os
from datetime import datetime, timedelta

class CVEDataFetcher:
    """Module to fetch CVE data from NVD or other CVE databases."""
    
    def __init__(self, cache_dir='.cve_cache'):
        self.cache_dir = cache_dir
        self.cache_file = os.path.join(cache_dir, 'cve_data.json')
        self.cache_expiry = timedelta(days=1)
        self.nvd_api_url = "https://services.nvd.nist.gov/rest/json/cves/1.0"
        
        if not os.path.exists(cache_dir):
            os.makedirs(cache_dir)
    
    async def fetch_cve_data(self, start_index=0, results_per_page=2000):
        """Fetch CVE data from NVD API asynchronously."""
        if self._is_cache_valid():
            return self._load_cache()
        
        all_cve_items = []
        
        async with aiohttp.ClientSession() as session:
            while True:
                params = {
                    'startIndex': start_index,
                    'resultsPerPage': results_per_page
                }
                
                try:
                    async with session.get(self.nvd_api_url, params=params) as response:
                        response.raise_for_status()
                        data = await response.json()
                
                    cve_items = data.get('result', {}).get('CVE_Items', [])
                    all_cve_items.extend(cve_items)
                
                    if len(cve_items) < results_per_page:
                        break
                
                    start_index += results_per_page
                except aiohttp.ClientError as e:
                    print(f"Error fetching CVE data: {e}")
                    break
        
        self._cache_data(all_cve_items)
        return all_cve_items
    
    def _is_cache_valid(self):
        """Check if the cache is still valid."""
        if not os.path.exists(self.cache_file):
            return False
        
        file_time = datetime.fromtimestamp(os.path.getmtime(self.cache_file))
        return datetime.now() - file_time < self.cache_expiry
    
    def _load_cache(self):
        """Load CVE data from cache."""
        with open(self.cache_file, 'r') as f:
            return json.load(f)
    
    def _cache_data(self, data):
        """Cache the fetched CVE data."""
        with open(self.cache_file, 'w') as f:
            json.dump(data, f)



import pytest
from unittest.mock import AsyncMock, patch
from cve_checker.vulnerability_checker import VulnerabilityChecker
from cve_checker.cve_data_fetcher import CVEDataFetcher

@pytest.fixture
def mock_cve_data():
    return [
        {
            'cve': {
                'CVE_data_meta': {'ID': 'CVE-2021-1234'},
                'description': {'description_data': [{'value': 'Test vulnerability'}]},
                'affects': {
                    'vendor': {
                        'vendor_data': [
                            {
                                'product': {
                                    'product_data': [
                                        {
                                            'product_name': 'test_software',
                                            'version': {
                                                'version_data': [
                                                    {'version_value': '1.0.0'}
                                                ]
                                            }
                                        }
                                    ]
                                }
                            }
                        ]
                    }
                }
            }
        }
    ]

@pytest.fixture
def vulnerability_checker():
    mock_fetcher = AsyncMock(spec=CVEDataFetcher)
    return VulnerabilityChecker(mock_fetcher)

@pytest.mark.asyncio
async def test_check_vulnerabilities(vulnerability_checker, mock_cve_data):
    vulnerability_checker.cve_data_fetcher.fetch_cve_data.return_value = mock_cve_data

    vulnerabilities = await vulnerability_checker.check_vulnerabilities('test_software', '1.0.0')

    assert len(vulnerabilities) == 1
    assert vulnerabilities[0]['cve_id'] == 'CVE-2021-1234'
    assert vulnerabilities[0]['description'] == 'Test vulnerability'
    assert vulnerabilities[0]['affected_version']['version_value'] == '1.0.0'

@pytest.mark.asyncio
async def test_check_vulnerabilities_no_match(vulnerability_checker, mock_cve_data):
    vulnerability_checker.cve_data_fetcher.fetch_cve_data.return_value = mock_cve_data

    vulnerabilities = await vulnerability_checker.check_vulnerabilities('other_software', '1.0.0')

    assert len(vulnerabilities) == 0

@pytest.mark.asyncio
async def test_check_vulnerabilities_version_mismatch(vulnerability_checker, mock_cve_data):
    vulnerability_checker.cve_data_fetcher.fetch_cve_data.return_value = mock_cve_data

    vulnerabilities = await vulnerability_checker.check_vulnerabilities('test_software', '2.0.0')

    assert len(vulnerabilities) == 0



import pytest
import aiohttp
from unittest.mock import patch, AsyncMock, MagicMock
from cve_checker.cve_data_fetcher import CVEDataFetcher

class AsyncContextManagerMock:
    def __init__(self, return_value):
        self.return_value = return_value

    async def __aenter__(self):
        return self.return_value

    async def __aexit__(self, exc_type, exc, tb):
        pass

@pytest.fixture
def cve_data_fetcher():
    return CVEDataFetcher()

@pytest.mark.asyncio
async def test_fetch_cve_data(cve_data_fetcher):
    mock_response = {
        'result': {
            'CVE_Items': [
                {'cve': {'CVE_data_meta': {'ID': 'CVE-2021-1234'}}},
                {'cve': {'CVE_data_meta': {'ID': 'CVE-2021-5678'}}}
            ]
        }
    }
    
    mock_response_obj = AsyncMock()
    mock_response_obj.json.return_value = mock_response
    mock_response_obj.raise_for_status = MagicMock()  # Change this to MagicMock

    mock_session = AsyncMock()
    mock_session.get = MagicMock(return_value=AsyncContextManagerMock(mock_response_obj))

    mock_client_session = AsyncContextManagerMock(mock_session)

    with patch('aiohttp.ClientSession', return_value=mock_client_session):
        with patch.object(cve_data_fetcher, '_is_cache_valid', return_value=False):
            with patch.object(cve_data_fetcher, '_cache_data'):
                result = await cve_data_fetcher.fetch_cve_data()
        
                assert len(result) == 2
                assert result[0]['cve']['CVE_data_meta']['ID'] == 'CVE-2021-1234'
                assert result[1]['cve']['CVE_data_meta']['ID'] == 'CVE-2021-5678'

def test_is_cache_valid(cve_data_fetcher, tmp_path):
    cve_data_fetcher.cache_file = tmp_path / "test_cache.json"
    
    # Test with no cache file
    assert not cve_data_fetcher._is_cache_valid()
    
    # Test with a fresh cache file
    cve_data_fetcher.cache_file.touch()
    assert cve_data_fetcher._is_cache_valid()

def test_load_cache(cve_data_fetcher, tmp_path):
    cve_data_fetcher.cache_file = tmp_path / "test_cache.json"
    test_data = [{'cve': {'CVE_data_meta': {'ID': 'CVE-2021-1234'}}}]
    
    cve_data_fetcher._cache_data(test_data)
    loaded_data = cve_data_fetcher._load_cache()
    
    assert loaded_data == test_data



import asyncio
from typing import List, Dict
from .cve_data_fetcher import CVEDataFetcher
from .vulnerability_checker import VulnerabilityChecker
from .dependency_scanner import DependencyScanner
from .risk_scorer import RiskScorer
from .report_generator import ReportGenerator

class CVEChecker:
    def __init__(self):
        self.cve_data_fetcher = CVEDataFetcher()
        self.vulnerability_checker = VulnerabilityChecker(self.cve_data_fetcher)
        self.dependency_scanner = DependencyScanner(self.cve_data_fetcher)
        self.risk_scorer = RiskScorer()
        self.report_generator = ReportGenerator()

    async def check_dependencies(self, dependency_file: str) -> str:
        vulnerabilities = await self.dependency_scanner.scan_dependencies(dependency_file)
        scored_vulnerabilities = self.risk_scorer.prioritize_vulnerabilities(vulnerabilities)
        report = self.report_generator.generate_report(scored_vulnerabilities, 'json')
        return report

    async def check_package(self, package_name: str, version: str) -> str:
        vulnerabilities = await self.vulnerability_checker.check_vulnerabilities(package_name, version)
        scored_vulnerabilities = self.risk_scorer.prioritize_vulnerabilities(vulnerabilities)
        report = self.report_generator.generate_report(scored_vulnerabilities, 'json')
        return report

def main():
    cve_checker = CVEChecker()
    
    # Example usage
    dependency_file = 'requirements.txt'
    report = asyncio.run(cve_checker.check_dependencies(dependency_file))
    print(f"Vulnerability report for {dependency_file}:")
    print(report)

    package_name = 'example_package'
    version = '1.0.0'
    report = asyncio.run(cve_checker.check_package(package_name, version))
    print(f"Vulnerability report for {package_name} version {version}:")
    print(report)

if __name__ == "__main__":
    main()



import json
import yaml
import toml
from typing import Dict, List
from .vulnerability_checker import VulnerabilityChecker
from .cve_data_fetcher import CVEDataFetcher

class DependencyScanner:
    def __init__(self, cve_data_fetcher: CVEDataFetcher):
        self.vulnerability_checker = VulnerabilityChecker(cve_data_fetcher)

    async def scan_dependencies(self, file_path: str) -> Dict[str, List[Dict]]:
        dependencies = self._parse_dependency_file(file_path)
        vulnerabilities = {}

        for package, version in dependencies.items():
            vulns = await self.vulnerability_checker.check_vulnerabilities(package, version)
            if vulns:
                vulnerabilities[package] = vulns

        return vulnerabilities

    def _parse_dependency_file(self, file_path: str) -> Dict[str, str]:
        if file_path.endswith('.txt'):
            return self._parse_requirements_txt(file_path)
        elif file_path.endswith('.json'):
            return self._parse_package_json(file_path)
        elif file_path.endswith('.yaml') or file_path.endswith('.yml'):
            return self._parse_yaml_file(file_path)
        elif file_path.endswith('.toml'):
            return self._parse_toml_file(file_path)
        else:
            raise ValueError(f"Unsupported file format: {file_path}")

    def _parse_requirements_txt(self, file_path: str) -> Dict[str, str]:
        dependencies = {}
        with open(file_path, 'r') as f:
            for line in f:
                if '==' in line:
                    package, version = line.strip().split('==')
                    dependencies[package] = version
        return dependencies

    def _parse_package_json(self, file_path: str) -> Dict[str, str]:
        with open(file_path, 'r') as f:
            data = json.load(f)
        return {**data.get('dependencies', {}), **data.get('devDependencies', {})}

    def _parse_yaml_file(self, file_path: str) -> Dict[str, str]:
        with open(file_path, 'r') as f:
            data = yaml.safe_load(f)
        return data.get('dependencies', {})

    def _parse_toml_file(self, file_path: str) -> Dict[str, str]:
        with open(file_path, 'r') as f:
            data = toml.load(f)
        return data.get('dependencies', {})



import argparse
import asyncio
from .cve_checker import CVEChecker

def main():
    parser = argparse.ArgumentParser(description="CVE Checker - Check for vulnerabilities in dependencies or specific packages")
    parser.add_argument("--dependencies", help="Path to dependency file (e.g., requirements.txt)")
    parser.add_argument("--package", help="Name of the package to check")
    parser.add_argument("--version", help="Version of the package to check")
    
    args = parser.parse_args()
    
    cve_checker = CVEChecker()
    
    if args.dependencies:
        report = asyncio.run(cve_checker.check_dependencies(args.dependencies))
        print(f"Vulnerability report for dependencies in {args.dependencies}:")
        print(report)
    elif args.package and args.version:
        report = asyncio.run(cve_checker.check_package(args.package, args.version))
        print(f"Vulnerability report for {args.package} version {args.version}:")
        print(report)
    else:
        parser.print_help()

if __name__ == "__main__":
    main()
